<?php
/**
 * Objet allows to get the informations of an object in order to use them in a template
 *
 * @package Publimmo
 * @subpackage Helpers
 */

namespace Classes\PublimmoPro;

/**
 * This class manages an Objet received from the webservice.
 *
 * @package Publimmo
 * @subpackage Helpers
 */
class Objet
{

    /** @var Client */
    public $client;

    /**
     * Object's data
     *
     * Usually retrieved from the webservice
     *
     * @var mixed
     */
    public $data;

    /**
     * Are we in a list or a detail page ?
     *
     * @var string
     **/
    public $env;

    /**
     * Define the page in which the object is shown : default, promo, front
     *
     * @var string
     **/
    public $displayType;

    /**
     * The webservices uses integers to define the types of objects. These constants
     * reflects what are these integers.
     */
    const APPARTEMENT_CODE = 1;
    const IMMEUBLE_CODE = 2;
    const COMMERCIAL_CODE = 3;
    const MAISON_CODE = 4;
    const TERRAIN_CODE = 5;
    const PARC_CODE = 6;
    const PROMOTION_CODE = 7;

    /**
     * Constructor of the Objet class
     *
     * Reads the args variable and merge with defaults.
     * Then it checks if an object_ID is provided -> go wp_remote_get the object.
     * or it will try to see if object_data is provided, which is used to force data
     * or it fails.
     *
     * @param mixed args
     **/
    public function __construct($args = null)
    {
        $defaults = array(
            'object_ID' => null,
            'object_data' => array(), // if there is no id, then it may be because we don't need to fetch if
            'displayType' => 'default',
            'no_transaction' => false
        );

        $this->client = new Client(get_option('publimmo_settings')['agency_id']);

        $r = wp_parse_args($args, $defaults);

        if (!empty($r['object_ID'])) {
            // Get remote data
            $this->data = $this->getRemoteData($r['object_ID']);
        } elseif (!empty($r['object_data'])) {
            // No need to get data, it's already been provided
            $this->data = $r['object_data'];
        } else {
            // Error
            error_log('['.get_bloginfo('name').'] Attributs object_ID ou object_data manquants');
            return false;
        }

        $this->displayType = $r['displayType'];
        $this->no_transaction = $r['no_transaction'];
        $this->env = (isset($this->data->titre)) ? 'detail': 'list';
    }

    /**
     * Get the objects informations from the webservice
     *
     * @param mixed $object_ID is the object id to fetch
     */
    public function getRemoteData($object_ID)
    {
        $url = $this->client->getApiUrl().'/objets/'.$object_ID.'/?key='.$this->client->getKey().'&ul='.ICL_LANGUAGE_CODE;
        $response = wp_remote_get($url);

        if (!is_array($response)) {
            error_log('['.get_bloginfo('name').'] Connection au webservice impossible');
            return false;
        }

        if ($response['response']['code'] == '404') {
            error_log('['.get_bloginfo('name').'] l\'objet '.$object_ID.' n\'existe pas.');
            return false;
        }

        return json_decode($response['body']);
    }

    /**
     * Get the title of the objects
     *
     * @param string $titleType allows to switch between to title types : normal and alternative
     **/
    public function getTitle($titleType = 'normal')
    {
        if ($titleType == 'normal') {
            if ($this->env == 'detail') {
                // if the title has to be displayed on front, remove transaction
                if ($this->no_transaction) {
                    return $this->data->titre;
                }

                return $this->data->transaction.' '.$this->data->titre;
            }

            if ($this->env == 'list') {
                if ($this->displayType == 'promo') {
                    $title = __("Promotion", "publimmo")." ".__("à", "publimmo")." ".$this->data->localite;
                } else {
                    if (in_array($this->data->typeCode, array(1,4))) { //Si Maison ou Appartement
                        $title = $this->data->type." ".$this->data->pieces." ".__("pièces", "publimmo")." ".__("à", "publimmo")." ".$this->data->localite;
                    } else {
                        $title = $this->data->type." ".__("à", "publimmo")." ".$this->data->localite;
                    }

                }

                // small french subtility
                $title = str_ireplace(array('à le ', 'à les '), array('au ','aux '), $title);

                // if the title has to be displayed on front, remove transaction
                if ($this->no_transaction) {
                    return $title;
                }

                return $this->data->transaction.' '.$title;
            }

        } elseif ($titleType == 'alternative') {
            //Returns the same title for both list and detail env
            return $this->data->soustitre;
        }
    }

    public function getTitlev2($titleType = 'normal'){

        $titleType = get_option('use_alternative_title','normal');
        //$titleType = 'normal';
        if ($titleType == 'normal' || $titleType == 'alternative_plus_title') {
            if ($this->env == 'detail') {
                if ($this->displayType == 'promo') {
                    $title = __("Promotion", "publimmo").' "' .$this->data->transaction.'" '.__(" à ", "publimmo").$this->data->localite;
                } else {
                    if(in_array($this->data->typeCode, array(1,4))) { //Si Maison ou Appartement
                        $title = $this->data->titre;
                    }else{
                        $title = $this->data->titre;
                    }
                }

                $title = str_ireplace(array('à le ', 'à les '), array('au ','aux '), $title);

                if ($titleType == 'alternative_plus_title') {
                    $title .= '<br><small>'.$this->data->soustitre.'</small>';
                }

                if ($this->no_transaction || $this->displayType == 'promo') {
                    return $title;
                }else{
                    return $this->data->transaction." ".$title;
                }

            }

            if ($this->env == 'list') {
                if ($this->displayType == 'promo' || ($this->displayType == 'default' && $this->data->type == 'Promotion')) {
                    $title = __("Promotion", "publimmo").' "' .$this->data->transaction.'" '.__(" à ", "publimmo").$this->data->localite;
                } else {
                    if (in_array($this->data->typeCode, array(1,4))){
                        $title = $this->data->localite." / ".$this->data->type.__(" de ", "publimmo").$this->data->pieces." ".__("pièces", "publimmo");
                    } else {
                        $title = $this->data->localite." / ".$this->data->type;
                    }
                }

                if ($titleType == 'alternative_plus_title') {
                    $title .= '<br><small>'.$this->data->soustitre.'</small>';
                }

                return $title;
            }
        } elseif ($titleType == 'alternative') {
            return $this->data->soustitre;
        }
    }

    /**
     * Get the price of the object
     *
     * @param string $currency allows to switch between eurochf and chf
     **/
    public function getPrice($currency = 'chf')
    {
        // Display "Liste de prix" only on the main promotion object, not the children objects
        if ($this->isPromotion() && $this->data->typeCode == self::PROMOTION_CODE) {
            return __('Liste de prix', 'publimmo');
        }

        if (isset($this->data->prix)) {
            if ($currency == "chf") {
                return "CHF ".$this->data->prix;
            } elseif ($currency == "eurochf") {
                return "CHF ".$this->data->prix." / EUR ~".$this->data->prixEuro;
            }
        } else {
            return __("Prix sur demande", "publimmo");
        }
    }

    /**
     * Get the price of the object
     *
     * @param string $currency allows to switch between eurochf and chf
     **/
    public function getPriceV2($currency = 'chf')
    {
        // Display "Liste de prix" only on the main promotion object, not the children objects
        if ($this->isPromotion() && $this->data->typeCode == self::PROMOTION_CODE) {
            return '<a href="#prop_promotion_table_title" class="js-scrollTo">'.__('Liste de prix', 'publimmo').'</a>';
        }

        if (isset($this->data->prix)) {
            if ($currency == "chf") {
                return "CHF ".$this->data->prix;
            } elseif ($currency == "eurochf") {
                return "CHF ".$this->data->prix." / EUR ~".$this->data->prixEuro;
            }
        } else {
            return __("Prix sur demande", "publimmo");
        }
    }

    /**
     * Check if the object is an object from the network
     */
    public function isFromNetwork() {
        // objects in the network have no contact name, but a courtierPrincipal
        return !isset($this->data->contact->name) && !empty($this->data->contact->courtierPrincipal);
    }

    /**
     * Check if the object is a promotion object
     **/
    public function isPromotion() {
        if (
            (isset($this->data->type) && $this->data->typeCode == self::PROMOTION_CODE)
            || isset($this->data->promotionId)
            || (isset($this->data->promotionTable) && count($this->data->promotionTable)>0)
            || isset($this->data->inPromotionTable)
        )
        {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Check if the object is of the given type using the typeCode field
     *
     * @param  string  $objectType type of object, must be one of appartement, immeuble, commercial, maison, terrain, parc, promotion
     *
     * @return boolean             whether the object's type is the one we've asked for
     */
    public function isType($objectType){
        //Just check that the typeCode is existing
        if (!defined('self::'. strtoupper($objectType).'_CODE')) {
            trigger_error($objectType.' is not a valid object type', E_USER_ERROR);
        }
        return $this->data->typeCode == constant('self::'. strtoupper($objectType).'_CODE');
    }

    /**
     * Get the url of the object
     **/
    public function getUrl() {
        if (!is_object($this->data)) {
            return '#';
        }
        $transaction = $loc = $desc='';
        if(isset($this->data->transaction)) {
            $transaction = sanitize_title($this->data->transaction).'/';
        }
        if(isset($this->data->localite)) {
            $loc = sanitize_title($this->data->localite).'/';
        }
        if(!$this->isPromotion()) {
            if(isset($this->data->descShort)) {
                $desc = sanitize_title($this->data->descShort).'/';
            } elseif(isset($this->data->desc)) {
                $desc = sanitize_title($this->data->desc).'/';
            }
        } else {
            /**
             * Special url for promotion type
             */
            $url_type_object = __('Promotion');
            if ($this->isType('promotion')) {
                $url_type_object = __('Promotion','publimmo');
            } elseif ($this->isType('appartement')) {
                $url_type_object = __('Appartement neuf','publimmo');
            } elseif ($this->isType('maison')) {
                $url_type_object = __('Maison neuve','publimmo');
            } elseif ($this->isType('commercial')) {
                $url_type_object = __('Locaux commerciaux neuf','publimmo');
            }
            $transaction .= sanitize_title($url_type_object).'/';
        }
        $desc = preg_replace('#([0-9]+)-([0-9]+)#','$1.$2',str_replace('m%c2%b2','m2',$desc));
        $homeUrl = home_url();
        $homeUrl = str_replace('?lang='.ICL_LANGUAGE_CODE,'',$homeUrl);
        $lang = '';
        if (substr($homeUrl,-1) == '/') {
            $homeUrl = substr($homeUrl,0,-1);
        }
        if (ICL_LANGUAGE_CODE != '' && ICL_LANGUAGE_CODE != 'fr') {
            $lang = '?lang='.ICL_LANGUAGE_CODE;
        }

        return $homeUrl.'/'.str_replace(' ','-',$transaction.$desc.$loc).$this->data->id.'/'.$lang;
    }

    /**
     * if the object is a promotion, get its children ids
     *
     */
    public function getPromotionsIDs()
    {
        if (!$this->isPromotion()) {
            return false;
        }

        if (empty($this->data->promotionTable)) {
            return false;
        }

        $ids = wp_list_pluck($this->data->promotionTable, 'id');
        return array_filter($ids, 'is_int');
    }
}
